import 'dart:io';

import 'package:ansicolor/ansicolor.dart';
import 'package:jt_flutter_cli/core/code/code_handler.dart';
import 'package:jt_flutter_cli/core/const/app_const.dart';
import 'package:jt_flutter_cli/core/const/command_const.dart';
import 'package:jt_flutter_cli/core/const/string_const.dart';
import 'package:jt_flutter_cli/core/path/path_handler.dart';
import 'package:jt_flutter_cli/core/util/model_util/helpers.dart';
import 'package:path/path.dart' as path;
import 'package:process_run/process_run.dart';
import 'package:yaml/yaml.dart';

class Utility {
  static void grantPermissions({
    required String folderPath,
  }) {
    if (Platform.isWindows) {
      Process.runSync('icacls', [folderPath, '/grant', 'Users:(OI)(CI)RX']);
    } else {
      Process.runSync('chmod', ['-R', 'u+rX', folderPath]);
    }
  }

  static String toPascalCase({
    required String input,
  }) {
    List<String> words = input.split(RegExp(r'[^a-zA-Z0-9]'));
    words.removeWhere((element) => element.isEmpty);
    return words
        .map((word) => word[0].toUpperCase() + word.substring(1).toLowerCase())
        .join('');
  }

  static String covertToObjVariableName({
    required String input,
  }) {
    List<String> words = toPascalCase(input: input).split('');
    words.removeWhere((element) => element.isEmpty);
    return words[0].toLowerCase() + words.join().substring(1);
  }

  static String camelCaseFirstLower({
    required String input,
  }) {
    if (input.isEmpty) {
      return '';
    }
    String camelCaseInput = camelCase(input);

    return camelCaseInput[0].toLowerCase() + camelCaseInput.substring(1);
  }

  static bool doesFileExist({
    required String filePath,
  }) {
    File file = File(filePath);
    return file.existsSync();
  }

  static Future<void> renameFilesWithSubstring({
    required String path,
    required String substring,
    required String newName,
  }) async {
    Directory(path).listSync(recursive: true).forEach((entity) async {
      if (entity is File) {
        String fileName =
            entity.path.split(Platform.pathSeparator).last.toLowerCase();
        if (fileName.contains(substring.toLowerCase())) {
          String newPath = entity.path
              .replaceAll(substring.toLowerCase(), newName.toLowerCase());
          await entity.rename(newPath);
        }
      }
    });
  }

  static Future<void> processFiles({
    required String path,
    required String substring,
    required String replacement,
    required bool isPascalCase,
    String targetFileName = '',
  }) async {
    String pascalCase = toPascalCase(input: replacement);
    List<FileSystemEntity> entities = Directory(path).listSync(recursive: true);
    for (var entity in entities) {
      if (entity is File) {
        if (targetFileName.isEmpty) {
          await processFile(
              file: entity,
              substring: substring,
              replacement: replacement,
              finalReplacement: isPascalCase ? pascalCase : replacement);
        } else {
          String fileName = entity.path.split(Platform.pathSeparator).last;
          if (fileName == targetFileName) {
            await processFile(
                file: entity,
                substring: substring,
                replacement: replacement,
                finalReplacement: isPascalCase ? pascalCase : replacement);
          }
        }
      }
    }
  }

  static Future<void> processFile({
    required File file,
    required String substring,
    required String replacement,
    required String finalReplacement,
  }) async {
    try {
      String content = await file.readAsString();
      content = content.replaceAll(substring, finalReplacement);
      await file.writeAsString(content);
    } catch (e) {
      Utility.errorPrinter(text: '$kErrorProcessingFile ${file.path}: $e');
    }
  }

  static String? getPubspecInfo() {
    try {
      String currentDirectory = Directory.current.path;
      String pubspecPath = '$currentDirectory$kPubspecPath';
      File pubspecFile = File(pubspecPath);
      String contents = pubspecFile.readAsStringSync();
      var yaml = loadYaml(contents);
      return yaml[kName];
    } catch (e) {
      Utility.errorPrinter(text: '$kErrorReadingPubspec $e');
      return null;
    }
  }

  static String getPackageNameAndVersion() {
    String packageName = kPackageName;
    String packageDirPath = Directory.current.path;
    File pubspecFile = File('$packageDirPath$kPubspecPath');
    if (!pubspecFile.existsSync()) {
      Utility.errorPrinter(text: '$kError $kPubspecNotFound');
      return '';
    }
    String pubspecContent = pubspecFile.readAsStringSync();
    var yaml = loadYaml(pubspecContent);
    var dependencies = yaml[kDependencies];
    if (dependencies != null && dependencies[packageName] != null) {
      var version = dependencies[packageName];
      String versionString = version.toString();
      if (versionString.startsWith('^')) {
        versionString = versionString.substring(1);
      }
      return '$packageName-$versionString';
    } else {
      Utility.errorPrinter(text: '$packageName $kDependencyNotInPubspec');
      return '';
    }
  }

  static Future<void> addImport({
    required String filePath,
    required List<String> importStatements,
  }) async {
    final file = File(filePath);
    final lines = await file.readAsLines();

    for (var importStatement in importStatements) {
      if (!lines.contains(importStatement)) {
        lines.insert(0, importStatement);
      }
    }
    await file.writeAsString(lines.join('\n'));
  }

  static Future<void> addCodeAboveBraces({
    required String filePath,
    required List<String> linesToAdd,
  }) async {
    final file = File(filePath);
    List<String> lines = await file.readAsLines();

    int closingBraceIndex = lines.lastIndexOf('}');
    if (closingBraceIndex == -1) {
      Utility.errorPrinter(text: '$kError $kClosingCurlyBracketNotFound');
      return;
    }
    lines.insertAll(closingBraceIndex, linesToAdd);
    await file.writeAsString(lines.join('\n'));
  }

  static Future<void> findAndReplace({
    required String filePath,
    required String searchString,
    required String replacement,
  }) async {
    try {
      final file = File(filePath);
      List<String> lines = await file.readAsLines();
      for (int i = 0; i < lines.length; i++) {
        if (lines[i].contains(searchString)) {
          lines[i] = lines[i].replaceAll(searchString, replacement);
        }
      }
      await file.writeAsString(lines.join('\n'));
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
  }

  Map<String, String> removeMapEntry(
      {required String keyToRemove, required Map<String, dynamic> map}) {
    if (keyToRemove != "" && map.containsKey(keyToRemove)) {
      map.remove(keyToRemove);
    }
    Map<String, String> convertedMap = {};
    map.forEach((key, value) {
      convertedMap[key] = value.toString();
    });
    return convertedMap;
  }

  String convertCamelcaseStringToUnderscore({required String input}) {
    return input.replaceAllMapped(RegExp('([A-Z])'),
        (Match match) => '_${match.group(0)?.toLowerCase()}');
  }

  String getFileName({required String filePath}) {
    return filePath.split('/').last;
  }

  static Future<void> appendLinesToFile({
    required String filePath,
    required List<String> linesToAppend,
  }) async {
    final file = File(filePath).openWrite(mode: FileMode.append);
    for (String line in linesToAppend) {
      file.write('$line\n');
    }
    await file.close();
  }

  static void createFolder({
    required String path,
  }) {
    Directory newFolder = Directory(path);

    if (!newFolder.existsSync()) {
      newFolder.createSync(recursive: true);
    }
  }

  static Future<void> copyFolder({
    required String sourcePath,
    required String destinationPath,
  }) async {
    Directory sourceDir = Directory(sourcePath);
    Directory destDir = Directory(destinationPath);
    if (!await destDir.exists()) {
      await destDir.create(recursive: true);
    }
    List<FileSystemEntity> contents = sourceDir.listSync();
    for (var entity in contents) {
      String newPath = path.join(destinationPath, path.basename(entity.path));
      if (entity is Directory) {
        await copyFolder(sourcePath: entity.path, destinationPath: newPath);
      } else if (entity is File) {
        await entity.copy(newPath);
      }
    }
  }

  static Future<void> formatCode() async {
    await Future.delayed(Duration(milliseconds: 1000));
    await Shell().run(kDartFormatCommand);
  }

  static void successPrinter({
    required String text,
    String? textWithoutColor,
  }) {
    AnsiPen pen = AnsiPen()..green(bold: true);
    print("${pen(text)} ${textWithoutColor ?? ""}");
  }

  static void infoPrinter({
    required String text,
    String? textWithoutColor,
  }) {
    AnsiPen pen = AnsiPen()..blue(bold: true);
    print("${pen(text)} ${textWithoutColor ?? ""}");
  }

  static void errorPrinter({
    required String text,
    String? textWithoutColor,
  }) {
    AnsiPen pen = AnsiPen()..red(bold: true);
    print("${pen(text)} ${textWithoutColor ?? ""}");
  }

  static String stdoutWrite({
    required String text,
    String? textWithoutColor,
  }) {
    AnsiPen pen = AnsiPen()..blue(bold: true);
    print("${pen(text)} ${textWithoutColor ?? ""}");
    return "${pen(text)} ${textWithoutColor ?? ""}";
  }

  static void commandError() {
    errorPrinter(text: kCouldNotFindCommand);
    infoPrinter(text: kAvailableCommands);
  }

  ///Pass path after feature
  static String getDirFromCliPackage({
    required String sourceFolderPath,
  }) {
    if (kIsDevMode) {
      return '$kJtFlutterCliPath$kFeature$sourceFolderPath';
    }
    String dependencyName = Utility.getPackageNameAndVersion();
    String dependencyPath = PathHandler().getDependencyPath();
    if (!Directory(dependencyPath).existsSync()) {
      Utility.errorPrinter(
          text: 'Dependency $dependencyName not found in .pub-cache directory');
      exit(1);
    }
    String dependencyLibPath = path.join(dependencyPath, 'lib');
    if (!Directory(dependencyLibPath).existsSync()) {
      Utility.errorPrinter(
          text: 'Dependency $dependencyName does not contain a lib folder');
      exit(1);
    }
    String localSourceFolderPath =
        '$dependencyLibPath$kFeature$sourceFolderPath';
    Utility.grantPermissions(folderPath: localSourceFolderPath);

    if (!Directory(localSourceFolderPath).existsSync()) {
      Utility.errorPrinter(
          text: '$kError $localSourceFolderPath folder does not exist');
      exit(1);
    }
    return localSourceFolderPath;
  }

  Future<void> addImportAndChangesInFile({
    required String filePath,
    required List<String> importStatements,
    required List<String> methodCalls,
  }) async {
    final file = File(filePath);
    final lines = await file.readAsLines();

    int lastImportIndex = -1;
    for (int i = 0; i < lines.length; i++) {
      if (lines[i].startsWith("$kImport ")) {
        lastImportIndex = i;
      } else {
        break;
      }
    }

    if (lastImportIndex != -1) {
      if (importStatements.isNotEmpty) {
        lines.insertAll(lastImportIndex + 1, importStatements);
      }
      int completionIndex = -1;
      for (int i = 0; i < lines.length; i++) {
        if (lines[i].trim() == '}') {
          completionIndex = i;
        }
      }

      if (completionIndex != -1) {
        if (methodCalls.isNotEmpty) {
          lines.insertAll(completionIndex, methodCalls);
        }
        await file.writeAsString(lines.join('\n'));
      }
    }
  }
}
