import 'dart:async';
import 'dart:io';

import 'package:jt_flutter_cli/core/code/code_handler.dart';
import 'package:jt_flutter_cli/core/const/string_const.dart';
import 'package:jt_flutter_cli/core/path/path_handler.dart';
import 'package:jt_flutter_cli/core/util/util.dart';

class CreateFeature {
  Future<void> createNewFeature({
    required String featureName,
  }) async {
    String destinationPath = '$kLibFeatures$featureName';

    String sourceFolderPath = Utility.getDirFromCliPackage(
      sourceFolderPath: kCreateFeatureBasicStructure,
    );

    Directory destinationDirectory = Directory(destinationPath);
    if (destinationDirectory.existsSync()) {
      Utility.errorPrinter(
        text: kErrorChooseDifferentFeatureName,
      );
      return;
    }

    try {
      await Utility.copyFolder(
        sourcePath: sourceFolderPath,
        destinationPath: destinationPath,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kStartComment,
        replacement: kWhitespaces,
        isPascalCase: false,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kEndComment,
        replacement: kWhitespaces,
        isPascalCase: false,
      );
      await Utility.renameFilesWithSubstring(
        path: destinationPath,
        substring: kBasicStructureLowerCase,
        newName: featureName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kBasicStructurePascalCase,
        replacement: featureName,
        isPascalCase: true,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kBasicStructureLowerCase,
        replacement: featureName.toLowerCase(),
        isPascalCase: false,
      );
      String? name = Utility.getPubspecInfo();
      if (name != null) {
        await Utility.processFiles(
          path: destinationPath,
          substring: kFlutterCleanArchitecture,
          replacement: name,
          isPascalCase: false,
        );
      }

      final nameOfFeature = featureName.toLowerCase();
      String pascalCase = Utility.toPascalCase(
        input: nameOfFeature,
      );

      await diGetterFileUpdate(
        nameOfFeature: nameOfFeature,
        pascalCase: pascalCase,
      );

      await mockDiGetterFileUpdate(
        nameOfFeature: nameOfFeature,
        pascalCase: pascalCase,
      );

      await prodDiGetterFileUpdate(
        nameOfFeature: nameOfFeature,
        pascalCase: pascalCase,
      );

      await injectionContainerFileUpdate(
        nameOfFeature: nameOfFeature,
        pascalCase: pascalCase,
      );

      Utility.successPrinter(
        text: '$featureName $kFeatureString $kCreated',
      );
    } catch (e) {
      Utility.successPrinter(text: '$kErrorCopyingFolder $e');
    }
  }

  ///update di getter file
  Future<void> diGetterFileUpdate({
    String nameOfFeature = kWhitespaces,
    String pascalCase = kWhitespaces,
  }) async {
    final diGetterImports = [
      PathHandler().getRepoImport(
        featureName: nameOfFeature,
      )
    ];
    final diGetterStatements = [
      kWhitespaces,
      CodeHandler().getDiGetterStatement(
        pascalCaseFeatureName: pascalCase,
      )
    ];
    await Utility().addImportAndChangesInFile(
      filePath: '$kLibCoreDi$kDiGetter',
      importStatements: diGetterImports,
      methodCalls: diGetterStatements,
    );
  }

  ///update mock di getter file
  Future<void> mockDiGetterFileUpdate({
    String nameOfFeature = kWhitespaces,
    String pascalCase = kWhitespaces,
  }) async {
    final mockDiImports = PathHandler().getMockDiImports(
      featureName: nameOfFeature,
    );

    List<String> mockDiGetterImports = [
      PathHandler().getRepoImport(
        featureName: nameOfFeature,
      )
    ];
    mockDiGetterImports.addAll(mockDiImports);
    final mockDiGetterStatements = [
      kWhitespaces,
      CodeHandler().getMockDiGetterStatement(
        pascalCaseFeatureName: pascalCase,
      ),
    ];
    await Utility().addImportAndChangesInFile(
      filePath: '$kLibCoreDi$kMockDiGetter',
      importStatements: mockDiGetterImports,
      methodCalls: mockDiGetterStatements,
    );
  }

  ///update prod di getter file
  Future<void> prodDiGetterFileUpdate({
    String nameOfFeature = kWhitespaces,
    String pascalCase = kWhitespaces,
  }) async {
    final prodDiGetterImports = [
      PathHandler().getProdDiGetterImport(
        featureName: nameOfFeature,
      ),
      PathHandler().getRepoImport(
        featureName: nameOfFeature,
      )
    ];
    final prodDiGetterStatements = [
      kWhitespaces,
      CodeHandler().getProdDiGetterStatement(
        pascalCaseFeatureName: pascalCase,
      )
    ];
    await Utility().addImportAndChangesInFile(
      filePath: '$kLibCoreDi$kProdDiGetter',
      importStatements: prodDiGetterImports,
      methodCalls: prodDiGetterStatements,
    );
  }

  ///update injection container file
  Future<void> injectionContainerFileUpdate({
    String nameOfFeature = kWhitespaces,
    String pascalCase = kWhitespaces,
  }) async {
    final injectionContainerImports =
        PathHandler().getInjectionContainerImports(
      featureName: nameOfFeature,
    );
    final injectionContainerStatements = [
      kWhitespaces,
      CodeHandler().getInjectionContainerStatement(
        pascalCaseFeatureName: pascalCase,
      )
    ];
    await Utility().addImportAndChangesInFile(
      filePath: '$kLibCoreDi$kInjectionContainer',
      importStatements: injectionContainerImports,
      methodCalls: injectionContainerStatements,
    );
  }
}
